'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
var hasNoHoles = function hasNoHoles(path) {
    return !!path.findParent(function (_ref) {
        var node = _ref.node;
        return node.directives && node.directives.some(function (_ref2) {
            var value = _ref2.value;
            return value.value === 'no holes';
        });
    });
};

var currier = function currier(curry, t) {
    return function (node) {
        return curry ? t.callExpression(t.identifier(curry), [node]) : node;
    };
};

exports.default = function (_ref3) {
    var t = _ref3.types;
    var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

    var curried = currier(options.curry, t);
    var isUnderscore = function isUnderscore(node) {
        return t.isIdentifier(node, { name: '_' });
    };

    return {
        visitor: {
            CallExpression: function CallExpression(path) {
                if (hasNoHoles(path)) {
                    return;
                }

                var parameters = [];
                if (t.isMemberExpression(path.node.callee) && isUnderscore(path.node.callee.object) || isUnderscore(path.node.callee)) {
                    parameters.push(path.scope.generateUidIdentifier('_'));
                }

                path.node.arguments.filter(isUnderscore).forEach(function (arg) {
                    parameters.push(path.scope.generateUidIdentifier('_'));
                });

                if (parameters.length === 0) {
                    return;
                }

                var provider = parameters.slice();
                var transformCallee = function transformCallee(callee) {
                    return isUnderscore(callee) ? provider.shift() : t.isMemberExpression(callee) && isUnderscore(callee.object) ? t.memberExpression(provider.shift(), callee.property, callee.computed) : callee;
                };

                var lambda = t.arrowFunctionExpression(parameters, t.callExpression(transformCallee(path.node.callee), path.node.arguments.map(function (arg) {
                    return isUnderscore(arg) ? provider.shift() : arg;
                })));

                path.replaceWith(curried(lambda));
            },
            MemberExpression: function MemberExpression(path) {
                if (hasNoHoles(path)) {
                    return;
                }

                var holes = [];

                if (isUnderscore(path.node.object)) {
                    holes.push(path.scope.generateUidIdentifier('_'));
                }

                if (isUnderscore(path.node.property)) {
                    holes.push(path.scope.generateUidIdentifier('_'));
                }

                if (holes.length === 0) {
                    return;
                }

                var provider = holes.slice();
                var transform = function transform(node) {
                    return isUnderscore(node) ? provider.shift() : node;
                };

                var lambda = t.arrowFunctionExpression(holes, t.memberExpression(transform(path.node.object), transform(path.node.property), path.node.computed));

                path.replaceWith(curried(lambda));
            },
            BinaryExpression: function BinaryExpression(path) {
                if (hasNoHoles(path)) {
                    return;
                }

                var parameters = [];

                if (isUnderscore(path.node.left)) {
                    parameters.push(path.scope.generateUidIdentifier('_'));
                }

                if (isUnderscore(path.node.right)) {
                    parameters.push(path.scope.generateUidIdentifier('_'));
                }

                if (parameters.length === 0) {
                    return;
                }

                var provider = parameters.slice();
                var transform = function transform(node) {
                    return isUnderscore(node) ? provider.shift() : node;
                };

                var lambda = t.arrowFunctionExpression(parameters, t.BinaryExpression(path.node.operator, transform(path.node.left), transform(path.node.right)));

                path.replaceWith(curried(lambda));
            },
            UnaryExpression: function UnaryExpression(path) {
                if (!t.isIdentifier(path.node.argument, { name: '_' }) || hasNoHoles(path)) {
                    return;
                }

                var parameter = path.scope.generateUidIdentifier('_');
                var lambda = t.arrowFunctionExpression([parameter], t.unaryExpression(path.node.operator, parameter, path.node.prefix));

                path.replaceWith(curried(lambda));
            },
            Identifier: function Identifier(path) {
                if (path.node.name !== '_' || hasNoHoles(path)) {
                    return;
                }

                var parameter = path.scope.generateUidIdentifier('_');
                var lambda = t.arrowFunctionExpression([parameter], parameter);

                path.replaceWith(curried(lambda));
            }
        }
    };
};
